$include (at89cx051.mcu)

; i2c
SCL     EQU     P3_3
SDA     EQU     P3_4
WP      EQU     P3_5

RELE    EQU     P3_7
HL      EQU     P3_2



;   
CODE_DEV_READY   EQU  005h   ; ,      
CODE_SERV_READY  EQU  010h   ; ,    ,  
CODE_DEV_FINALY  EQU  008h   ; ,      
CODE_DEV_OK      EQU  027h   ;    ,  
CODE_MODE_READ   EQU  001h   ; ,      
CODE_MODE_WRITE  EQU  002h   ; ,      
CODE_MODE_PLAY   EQU  003h   ; ,      
CODE_MODE_NOTE   EQU  004h   ; ,      
CODE_MODE_OFF    EQU  005h   ; ,      


WRITE_PAGE_SIZE  EQU  64     ;  ,   

HL_UART_COUNT    EQU  32     ;   /    

CHIP_ADDRES_1    EQU  10100000b


STACK            EQU 056h

;  
NOTE_T2_HI         EQU   10   ;    ( )
NOTE_T2_LO         EQU   11   ; -//-  
NOTE_VOLUME        EQU   12   ;    
NOTE_DURABILITY    EQU   13   ;    ( )    50 
HL_COUNTER         EQU   14   ;    





CSEG AT 0
;    
	  ljmp     start

ORG    0Bh
;    T/C0
    ljmp    interrupt_handler_tc0

ORG    30h

start:
;  
     setb    RELE
     clr     HL
;  
     mov     SP, #STACK
;     
     mov     DPTR, #d_notes
;   
     lcall   serial_init
     
     mov     NOTE_VOLUME, #255
     mov     HL_COUNTER, #HL_UART_COUNT



; ,      ..
    mov      A, #100                       ; A -     
verify_downloader_cycle:
    mov      B, #100                       ; B -    
verify_downloader_wait:
    djnz     B, verify_downloader_wait
    jnb      RXD, downloader_found
    dec      A
    jnz      verify_downloader_cycle




    

play_start:
    mov      A, #CHIP_ADDRES_1
    lcall    play_melody

power_off:
    clr      RELE
repeat:
    jmp      repeat



;        
downloader_found:

    setb    HL

    ;    
    mov     A, #CODE_DEV_READY
    lcall   serial_send
    
    ;     
downloader_wait_ready:
    lcall   serial_read
    lcall   serial_send
    cjne    A, #CODE_SERV_READY, downloader_wait_ready

    ;    
    mov     A, #CODE_DEV_FINALY
    lcall   serial_send

    
    ;        
downloader_menu:
;    clr     HL
    ;   
    lcall   serial_read
    ;    
    lcall   serial_send
    cjne    A, #CODE_MODE_READ, downloader_menu_verify_write
    jmp     loader_send_data
downloader_menu_verify_write:
    cjne    A, #CODE_MODE_WRITE, downloader_menu_verify_play
    jmp     loader_flash_data
downloader_menu_verify_play:
    cjne    A, #CODE_MODE_PLAY, downloader_menu_verify_note
    jmp     loader_play_data
downloader_menu_verify_note:
    cjne    A, #CODE_MODE_NOTE, downloader_menu_verify_off
    jmp     loader_play_note
downloader_menu_verify_off:
    cjne    A, #CODE_MODE_OFF, downloader_menu
    jmp     loader_power_off
    
    




;-------------------------------------------------
;      
loader_send_data:
    ;   
    lcall   serial_read
    mov     R1, A
    lcall   i2c_set_addres
    ;   
    lcall   serial_read
    mov     R2, A                             ; R2 -    
    mov     R3, #0FFh                         ; R3 -      

    lcall   i2c_start
    mov     A, R1
    inc     A
    lcall   i2c_out
    lcall   i2c_wait_ack

loader_send_cycle:
    lcall   i2c_in
    lcall   serial_send
    lcall   i2c_send_ack
    
    djnz    R3, loader_send_cycle             ;    
    mov     R3, #0FFh
    djnz    R2, loader_send_cycle             ;   


    ;    i2c
    ; ..      ACK,     
    lcall   i2c_in
    ;    NOACK + STOP
    lcall   i2c_send_noack
    lcall   i2c_stop

    jmp     downloader_menu             ;      


;-------------------------------------------
;     
loader_flash_data:
    ;       
    clr     WP
    ;   
    lcall   serial_read
    mov     R1, A
    lcall   i2c_start
    lcall   i2c_out
    lcall   i2c_wait_ack
    ;    
    lcall   serial_read
    lcall   i2c_out
    lcall   i2c_wait_ack
    ;    
    lcall   serial_read
    lcall   i2c_out
    lcall   i2c_wait_ack

    ;   
    mov     R2, #WRITE_PAGE_SIZE

    
loader_flash_cycle:
    lcall   serial_read
    lcall   i2c_out
    lcall   i2c_wait_ack

    djnz    R2, loader_flash_cycle
    
    lcall   i2c_stop

    ;                
    ;       START        ,    
    mov     A, R1
    lcall   i2c_wait_writing

    ;       
    setb    WP

    jmp     downloader_menu             ;      
    
    
;----------------------------------
;   
loader_play_data:
    ;   
    lcall   serial_read
    lcall   play_melody
    
    ;    
    mov     A, #CODE_DEV_OK
    lcall   serial_send

    jmp     downloader_menu             ;      


;-------------------
;  
loader_play_note:
    ;        
    mov     HL_COUNTER, #HL_UART_COUNT
    ;   
    lcall   serial_read
    mov     R1, A
    ;   
    lcall   serial_read
    
    lcall   play_note
    
    ;    
    mov     A, #CODE_DEV_OK
    lcall   serial_send
    mov     HL_COUNTER, #HL_UART_COUNT
    jmp     downloader_menu             ;      


;-------------------
;  
loader_power_off:
    jmp     power_off

$include (i2c.a51)
$include (serial.a51)





; ============================================================================
;  
;      A
play_melody:
    ;   
    mov     R1, A

    lcall   i2c_set_addres


    lcall   i2c_start
    mov     A, R1
    inc     A

    lcall   i2c_out
    lcall   i2c_wait_ack
    
    
    
;  
    jmp     play_notes_cycle_go ;     
    
play_notes_cycle:
    lcall   i2c_send_ack
play_notes_cycle_go:            ;     
    lcall   i2c_in              ; 0 - , 0xFF -  ,  - 
    mov     R1, A
    lcall   i2c_send_ack
    lcall   i2c_in              ;  .  0 -   
    jz      play_notes_done
    
    lcall   play_note

    jmp     play_notes_cycle
play_notes_done:


    
;  WAV-
    ;   R1  
    lcall   i2c_send_ack
    lcall   i2c_in
    mov     R1, A


play_wav_cycle:
    lcall   i2c_send_ack
    lcall   i2c_in
    mov     P1, A             ;    
    

    ;    0xE0,   
    cjne    A, #0E0h, play_wave_cmp
play_wave_cmp:
    jc      play_wave_not_inverse
    cpl     HL
play_wave_not_inverse:


    ;  
    mov     B, R1
play_wav_wait_cycle:
    djnz    B, play_wav_wait_cycle

    jnz     play_wav_cycle
    
    
    
;   
play_notes_cycle_2:
    lcall   i2c_send_ack
    lcall   i2c_in              ; 0 - , 0xFF -  ,  - 
    mov     R1, A
    lcall   i2c_send_ack
    lcall   i2c_in              ;  .  0 -   
    jz      play_notes_done_2

    lcall   play_note

    jmp     play_notes_cycle_2
play_notes_done_2:


    
    
    mov     P1, #0
    
    lcall   i2c_send_noack
    lcall   i2c_stop
    
    ret
    
    
; ============================================================================
;         (NOTE_T2_HI, NOTE_T2_LO)
;  : 12 + 2*NOTE_T2_LO                                ( NOTE_T2_HI == 0)
;                 10 + 2*NOTE_T2_LO + 514*NOTE_T2_HI               ( NOTE_T2_HI != 0)
; NOTE_T2_LO    0!
delay_t2:
    ;    NOTE_T2_HI * 255 
    mov     R1, NOTE_T2_HI
    cjne    R1, #0, delay_t2_hi
    jmp     delay_t2_no_hi
delay_t2_hi:
    mov     B, #255
delay_t2_255_loop:
    djnz    B, delay_t2_255_loop
    djnz    R1, delay_t2_hi
delay_t2_no_hi:
    mov     B, NOTE_T2_LO
delay_t2_lo:
    djnz    B, delay_t2_lo
    ret
; ============================================================================



; ============================================================================
;  
;     R1    (0-, 0xFF- )
;     A    (  ,  R1==0xFF)
play_note:
    ;   
    cpl     HL

    ; ,     
    cjne    R1, #0FFh, play_note_note
    ;     
    mov     NOTE_VOLUME, A
    ret
    ;   
play_note_note:


    ;     
    mov     R2, A                       ;     R2
    mov     A, R1
    rl      A                           ; A = A * 2
    movc    A, @A + DPTR
    mov     NOTE_T2_HI, A
    mov     A, R1
    rl      A
    inc     A                           ; A = A * 2 + 1
    movc    A, @A + DPTR
    mov     NOTE_T2_LO, A

    mov     TL0, #LOW(NOT(25000-1))     ;  
    mov     TH0, #HIGH(NOT(25000-1))    ;   25 
    setb    EA                          ;  
    setb    ET0                         ;    /0
    mov     R3, SP                      ;     
    setb    TR0                         ;  /0
    
    cjne    R1, #0, play_note_cycle
play_pause_cycle:
    mov     P1, #0
    jmp     play_pause_cycle

play_note_cycle:
    mov     P1, NOTE_VOLUME
    lcall   delay_t2
    mov     P1, #0
    lcall   delay_t2
    jmp     play_note_cycle
    
; ,           ,
;   ret   

    
    
; *************************************************************************************************
;   
; *************************************************************************************************
d_notes:
  $include (notes.a51)


; *************************************************************************************************
;    T/C0
; *************************************************************************************************
interrupt_handler_tc0:
    dec     R2
    cjne    R2, #0, interrupt_handler_reinit
    clr     TR0 ; TCON.4                ;  T/C0
    mov     SP, R3                      ;  
    reti
interrupt_handler_reinit:
    mov     TL0, #LOW(NOT(25000-1))     ;  
    mov     TH0, #HIGH(NOT(25000-1))    ;   25 
    reti


; ============================================================================
;      ,   
count_and_inverse_hl:
    djnz    HL_COUNTER, count_and_inverse_hl_ret
    mov     HL_COUNTER, #HL_UART_COUNT
    cpl     HL
count_and_inverse_hl_ret:
    ret

; ============================================================================

END


