; ============================================================================
; Sending a START condition
; return: set C if error
i2c_start:
    setb    SDA
    setb    SCL
    jnb     SCL, i2c_start_error    ; i2c bus is busy
    jnb     SDA, i2c_start_error    ; i2c bus is busy

    clr     SDA
    nop                             ; 1uS delay
    nop                             ; 1uS delay
    clr     SCL

    clr     C                       ; reset error flag
    ret
i2c_start_error:
    setb	  C                       ; set error flag
    ret
; ============================================================================



; ============================================================================
; Sending a STOP condition
i2c_stop:
    clr     SDA
    nop
    nop
    setb    SCL
    nop
    nop
    setb	  SDA
    ret
; ============================================================================



; ============================================================================
; Sending a NO ACKNOWLEDGE condition
i2c_send_noack:
    setb    SDA
    nop
    nop
i2c_SCL_pulse:
    setb	  SCL
    nop
    nop
    clr     SCL
    ret
; ============================================================================


; ============================================================================
; Sending a ACKNOWLEDGE condition
i2c_send_ack:
    clr     SDA
    nop
    nop
    sjmp		i2c_SCL_pulse
; ============================================================================


; ============================================================================
; ,       ACKNOWLEDGE
i2c_wait_ack:
    setb    SDA
    nop
    nop
    setb    SCL
i2c_wait_ack_loop:
;    nop
;    nop
    jb      SDA, i2c_wait_ack_loop
    clr     SCL
    ret
; ============================================================================


; ============================================================================
; Sending byte from A to i2c bus
i2c_out:
    mov     B, #8                  ; bit counter
i2c_out_loop:
    rlc     A                      ; hi bit -> C flag
    mov     SDA, C
    nop
    nop
    setb	  SCL	                   ; strobe
    nop
    nop
    clr	    SCL
    djnz    B, i2c_out_loop        ; goto next bit
    ret
; ============================================================================


; ============================================================================
; Getting byte from i2c to A
i2c_in:
    mov     B, #8                  ; bit counter
    setb    SDA                    ; prepare to reading
i2c_in_loop:
    setb    SCL                    ; strobe
    nop
    nop
    mov     C, SDA
    rlc     A                      ; C -> low bit
    clr     SCL                    ; end of strobe
    djnz    B, i2c_in_loop         ; goto next bit
    ret
; ============================================================================


; ============================================================================
;    /
; :
;		A -  SLAVE-
i2c_set_addres:
    lcall    i2c_start
    lcall    i2c_out

    lcall    i2c_wait_ack
    mov      A, #0
    lcall    i2c_out
    lcall    i2c_wait_ack

    mov      A, #0
    lcall    i2c_out
    lcall    i2c_wait_ack

    ret
; ============================================================================


; ============================================================================
;   .      , 
;   
;
; :
;		A -  SLAVE-
i2c_wait_writing:
    mov      B, R1
    push     B
    mov      R1, A
i2c_wait_writing_loop:
    mov      A, R1
    lcall    i2c_start
    lcall    i2c_out

    ;    
    setb     SDA
    nop
    nop
    setb     SCL
    ; ..
    mov      B, #255
i2c_wait_writing_waiting:
    nop
    jnb      SDA, i2c_wait_writing_done
    djnz     B, i2c_wait_writing_waiting
    ;   ,    
    clr      SCL
    lcall    i2c_stop
    jmp      i2c_wait_writing_loop
i2c_wait_writing_done:
    clr      SCL
    lcall    i2c_stop
    pop      B
    mov      R1, B
    ret

; ============================================================================


